/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.xnio.metadata;

import java.io.Serializable;
import java.util.List;
import java.util.ArrayList;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlType;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlAttribute;

/**
 *
 */
@XmlType(name = "sslTcpConnectorType", propOrder = {})
public final class SslTcpConnectorMetaData implements Serializable, BindableMetaData, OptionedMetaData {

    private static final long serialVersionUID = 2101881740511543307L;

    private SocketAddressMetaData binding;
    private List<OptionMetaData> options = arrayList();
    private OptionMetaData option;
    private RefMetaData executor;
    private RefMetaData sslContext;
    private RefMetaData sslExecutor;
    private String name;

    private static <T> List<T> arrayList() {
        return new ArrayList<T>();
    }

    public RefMetaData getExecutor() {
        return executor;
    }

    @XmlElement(name = "executor")
    public void setExecutor(final RefMetaData executor) {
        this.executor = executor;
    }

    public SocketAddressMetaData getBinding() {
        return binding;
    }

    @XmlElement(name = "bind-address")
    public void setBinding(final SocketAddressMetaData binding) {
        this.binding = binding;
    }

    public List<OptionMetaData> getOptions() {
        return options;
    }

    @XmlElementWrapper(name = "options")
    @XmlElement(name = "option")
    public void setOptions(final List<OptionMetaData> options) {
        this.options = options;
    }

    public OptionMetaData getOption() {
        return option;
    }

    @XmlElement
    public void setOption(final OptionMetaData option) {
        this.option = option;
    }

    public RefMetaData getSslContext() {
        return sslContext;
    }

    @XmlElement(name = "ssl-context")
    public void setSslContext(final RefMetaData sslContext) {
        this.sslContext = sslContext;
    }

    public RefMetaData getSslExecutor() {
        return sslExecutor;
    }

    @XmlElement(name = "ssl-executor")
    public void setSslExecutor(final RefMetaData sslExecutor) {
        this.sslExecutor = sslExecutor;
    }

    public String getName() {
        return name;
    }

    @XmlAttribute(required = true)
    public void setName(final String name) {
        this.name = name;
    }
}